# MIT License
#
# Copyright (c) 2018 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import logging
from collections import defaultdict

from sortedcontainers import SortedList

from base.executors.batch_work_executor import BatchWorkExecutor
from base.jobs.base_job import BaseJob
from cli.check_telnet import  check_port, line_to_host_port
from cli.helper import combine_list_number

logger = logging.getLogger(__name__)


class MultiThreadsTelnetJob(BaseJob):
    def __init__(
            self,
            work_iterable,
            max_workers=32,
            batch_size=1):
        self.work_iterable = work_iterable
        self.batch_work_executor = BatchWorkExecutor(batch_size, max_workers)
        self._dict_cache = []
        self.max_workers=max_workers
        self.list_open = defaultdict(SortedList)
        self.list_not_open = defaultdict(SortedList)

    def _start(self):
        # self.item_exporter.open()
        pass

    def _export(self):
        host_port_list = []
        # separate to host port pair
        for line in self.work_iterable:
            line =line.strip()
            if line.startswith("#"):
                continue
            host, ports, _range = line_to_host_port(line)
            for port in ports:
                host_port_list.append((host, port))

        self.batch_work_executor.execute(
            host_port_list,
            self._export_batch,
            total_items=len(host_port_list)
        )

    def _export_batch(self, work_data):
        for line in work_data:
            host, port = line
            ok = check_port(host,port)
            if ok:
                self.list_open[host].add(int(port))
            else:
                self.list_not_open[host].add(int(port))


    def _end(self):
        self.batch_work_executor.shutdown()
        # self.item_exporter.close()
        pass

    def get_cache(self):
        return self._dict_cache

    def clean_cache(self):
        self._dict_cache = []
    def get_result(self):
        for host in self.list_open:
            self.list_open[host] = combine_list_number(self.list_open[host])
        for host in self.list_not_open:
            self.list_not_open[host] = combine_list_number(self.list_not_open[host])

        return {
            "open": self.list_open,
            "close": self.list_not_open
        }



import socket

def check_port(host='10.1.16.211',port="11107"):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(3)
    result = sock.connect_ex((host,int(port)))
    ok =True
    if result == 0:
       # print( f"Host {host} Port {port} is open")
       pass
    else:
       print (f"Host {host} Port {port} not open")
       ok = False
    sock.close()
    return ok

def check_ports (host,ports):
    for port in ports:
        check_port(host,port)

def create_list(r1, r2):
    return list(range(r1,r2+1))

def line_to_host_port(line:str):
    host,ports_string = line.strip().split(",")
    _range =False
    if " " in ports_string:
       ports = ports_string.split(" ")
    elif "-" in ports_string:
        start, end = ports_string.split("-")
        ports = create_list(int(start),int(end))
        _range = True
    else:
        ports = [ports_string]
    return host, ports, _range

# host, ports  = line_to_host_port("10.1.44.16,8020 585")


# with open("example_check_port_list","r") as f:
#     lines = f.readlines()
#     for line in lines:
#         host,ports = line_to_host_port(line)
#         check_ports(host,ports)





import setuptools

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setuptools.setup(
    name="Help-Tool-ThanhNv",
    version="1.1.4",
    author="LinLin",
    author_email="nguyenthanh2303@gmail.com",
    description="",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/thanhnv2303/HelpTool",
    project_urls={
        "Bug Tracker": "https://github.com/thanhnv2303/HelpTool",
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ],
    keywords='checkport',
    python_requires='>=3.7.2,<4',
    packages= setuptools.find_packages(exclude=['schemas', 'tests']),
    install_requires=[
        'requests<=2.26.0',
        'click>=8.0.4,<9',
        "sortedcontainers==2.4.0"

    ],
    entry_points={
        'console_scripts': [
            'help_tool=cli:cli',
        ],
    },
)


def combine_list_number(list_number):

    if(len(list_number)==0):
        return list_number
    start = list_number[0]
    end = start
    new_list = []
    for number in list_number[1:-1]:
        if number == end+1:
            end = number
        elif start == end:
            new_list.append(str(end))
            start = number
            end = number
        else:
            new_list.append(f"{start}-{end}")
            start = number
            end = number

    if list_number[-1] == end + 1:
        new_list.append(f"{start}-{list_number[-1]}")
    elif start!= list_number[-1]:
        new_list.append(str(start))
        new_list.append(str(list_number[-1]))
    else:
        new_list.append(str(list_number[-1]))
    return new_list
